import subprocess

class Terraform:

    def show(self, plan_file):
        try:
            proc = subprocess.run([
                'terraform',
                'show',
                '-no-color',
                plan_file,
            ], capture_output=True)
            plan_output = proc.stdout
            if proc.stderr:
                plan_output = proc.stderr
        except subprocess.CalledProcessError as exc:
            plan_output = exc.output

        return plan_output.decode()



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='terratalk',
    version='0.0.4',
    author='Gunter Grodotzki',
    author_email='gunter@grodotzki.co.za',
    description='Terraform commentator.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/lifeofguenter/terratalk',
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3',
    install_requires=[
        'click',
        'requests',
    ],
    entry_points={
        'console_scripts': [
            'terratalk = terratalk.__main__:cli',
        ],
    },
)



import requests
import urllib.parse
from time import sleep


class BitbucketServer:
    def __init__(self, base_url, username, password):
        self.base_url = base_url
        self.username = username
        self.password = password


    def pr(self, project_key=None, repository_slug=None, pull_request_id=None):
        self.project_key = project_key
        self.repository_slug = repository_slug
        self.pull_request_id = pull_request_id


    def comments(self):
        request_url = f'{self.base_url}/rest/api/1.0/projects/{self.project_key}/repos/{self.repository_slug}/pull-requests/{self.pull_request_id}/activities'
        comments = []
        for value in self.paginator(request_url):
            if value['action'] == 'COMMENTED':
                comments.append(value)

        return comments


    def comment_add(self, comment):
        request_url = f'{self.base_url}/rest/api/1.0/projects/{self.project_key}/repos/{self.repository_slug}/pull-requests/{self.pull_request_id}/comments'
        r = requests.post(request_url, json={'text': comment}, auth=(self.username, self.password))
        return r

    def comment_delete(self, comment_id, version):
        request_url = f'{self.base_url}/rest/api/1.0/projects/{self.project_key}/repos/{self.repository_slug}/pull-requests/{self.pull_request_id}/comments/{comment_id}'
        r = requests.delete(request_url, params={'version': version}, auth=(self.username, self.password))
        return r


    def paginator(self, url, params={}):
        values = []
        retries = 0

        while True:
            r = requests.get(url, params=params, auth=(self.username, self.password))
            buf = r.json()

            if 'values' not in buf and retries < 3:
                print(buf)
                retries = retries + 1
                sleep(1)
                continue

            values += buf['values']

            if buf['isLastPage']:
                break
            else:
                params['start'] = buf['nextPageStart']

            retries = 0

        return values



import os
import re
import subprocess

import click

from terratalk.bitbucket_server import BitbucketServer
from terratalk.terraform import Terraform


@click.group()
def cli():
    pass


@cli.command()
@click.option('-s', '--server')
@click.option('-u', '--username')
@click.option('-p', '--password')
@click.option('-w', '--workspace')
@click.option('--project-key')
@click.option('--repository-slug')
@click.option('--pull-request-id')
def comment(server, username, password, workspace, project_key, repository_slug, pull_request_id):

    if server is None and project_key is None and repository_slug is None and pull_request_id is None and os.getenv('CHANGE_URL') is not None:
        m = re.match(r'\A(https?://.*?)/projects/([^/]+)/repos/([^/]+)/pull-requests/(\d+)', os.getenv('CHANGE_URL'), re.IGNORECASE)
        server = m.group(1)
        project_key = m.group(2)
        repository_slug = m.group(3)
        pull_request_id = m.group(4)

    if username is None and os.getenv('STASH_USER') is not None:
        username = os.getenv('STASH_USER')

    if password is None and os.getenv('STASH_PASS') is not None:
        password = os.getenv('STASH_PASS')

    bs = BitbucketServer(base_url=server, username=username, password=password)
    bs.pr(project_key=project_key, repository_slug=repository_slug, pull_request_id=pull_request_id)

    # delete any older comments
    for comment in bs.comments():
        if comment['comment']['text'].lstrip().startswith(f'[comment]: # (terratalk: {workspace})'):
            click.echo('[terratalk] deleting comment {comment_id}'.format(comment_id=comment['id']))
            bs.comment_delete(comment['comment']['id'], comment['comment']['version'])

    # fetch terraform output
    tf = Terraform()
    plan_output = tf.show(workspace + '.plan')

    if plan_output == '' or plan_output.lstrip().startswith('This plan does nothing.'):
        print('[terratalk] this plan does nothing')
        exit()

    # https://bitbucket.org/tutorials/markdowndemo/issues/15/how-can-you-insert-comments-in-the#comment-22433250
    bs.comment_add(f'''
[comment]: # (terratalk: {workspace})
### tf plan output: {workspace}
```
{plan_output}
```
''')


if __name__ == '__main__':
    cli()
